from pysimplesat.endpoints.base.base_endpoint import SimpleSatEndpoint
from pysimplesat.interfaces import (
    IGettable,
    IPuttable
)
from pysimplesat.models.simplesat import Customer
from pysimplesat.types import (
    JSON,
    SimpleSatRequestParams,
)


class CustomersIdEndpoint(
    SimpleSatEndpoint,
    IGettable[Customer, SimpleSatRequestParams],
    IPuttable[Customer, SimpleSatRequestParams],
):
    def __init__(self, client, parent_endpoint=None) -> None:
        SimpleSatEndpoint.__init__(self, client, "{id}", parent_endpoint=parent_endpoint)
        IGettable.__init__(self, Customer)
        IPuttable.__init__(self, Customer)

    def get(
        self,
        data: JSON | None = None,
        params: SimpleSatRequestParams | None = None,
    ) -> Customer:
        """
        Performs a GET request against the /customers/{id} endpoint.

        Parameters:
            data (dict[str, Any]): The data to send in the request body.
            params (dict[str, int | str]): The parameters to send in the request query string.
        Returns:
            AuthInformation: The parsed response data.
        """
        return self._parse_one(
            Customer,
            super()._make_request("GET", data=data, params=params).json(),
        )

    def put(
        self,
        data: JSON | None = None,
        params: SimpleSatRequestParams | None = None,
    ) -> Customer:
        """
        Performs a PUT request against the /customers/{id} endpoint.

        Parameters:
            data (dict[str, Any]): The data to send in the request body.
            params (dict[str, int | str]): The parameters to send in the request query string.
        Returns:
            Customer: The parsed response data.
        """
        return self._parse_one(
            Customer,
            super()._make_request("PUT", data=data, params=params).json(),
        )



from pysimplesat.endpoints.base.base_endpoint import SimpleSatEndpoint
from pysimplesat.endpoints.simplesat.ResponsesIdEndpoint import ResponsesIdEndpoint
from pysimplesat.endpoints.simplesat.ResponsesSearchEndpoint import ResponsesSearchEndpoint
from pysimplesat.endpoints.simplesat.ResponsesCreateOrUpdateEndpoint import ResponsesCreateOrUpdateEndpoint


class ResponsesEndpoint(
    SimpleSatEndpoint,
):
    def __init__(self, client, parent_endpoint=None) -> None:
        SimpleSatEndpoint.__init__(self, client, "responses", parent_endpoint=parent_endpoint)
        self.search = self._register_child_endpoint(ResponsesSearchEndpoint(client, parent_endpoint=self))
        self.createorupdate = self._register_child_endpoint(ResponsesCreateOrUpdateEndpoint(client, parent_endpoint=self))

    def id(self, id: int) -> ResponsesIdEndpoint:
        """
        Sets the ID for this endpoint and returns an initialized ResponsesIdEndpoint object to move down the chain.

        Parameters:
            id (int): The ID to set.
        Returns:
            ResponsesIdEndpoint: The initialized ResponsesIdEndpoint object.
        """
        child = ResponsesIdEndpoint(self.client, parent_endpoint=self)
        child._id = id
        return child



from abc import ABC, abstractmethod
from typing import TYPE_CHECKING, Generic, TypeVar

from pysimplesat.responses.paginated_response import PaginatedResponse
from pysimplesat.types import (
    JSON,
    SimpleSatRequestParams,
    PatchRequestData,
)

if TYPE_CHECKING:
    from pydantic import BaseModel

TModel = TypeVar("TModel", bound="BaseModel")
TRequestParams = TypeVar(
    "TRequestParams",
    bound=SimpleSatRequestParams,
)


class IMethodBase(ABC, Generic[TModel, TRequestParams]):
    def __init__(self, model: TModel) -> None:
        self.model = model


class IPaginateable(IMethodBase, Generic[TModel, TRequestParams]):
    def __init__(self, model: TModel) -> None:
        super().__init__(model)

    @abstractmethod
    def paginated(
        self,
        page: int,
        page_size: int,
        params: TRequestParams | None = None,
    ) -> PaginatedResponse[TModel]:
        pass


class IGettable(IMethodBase, Generic[TModel, TRequestParams]):
    def __init__(self, model: TModel) -> None:
        super().__init__(model)

    @abstractmethod
    def get(
        self,
        data: JSON | None = None,
        params: TRequestParams | None = None,
    ) -> TModel:
        pass


class IPostable(IMethodBase, Generic[TModel, TRequestParams]):
    def __init__(self, model: TModel) -> None:
        super().__init__(model)

    @abstractmethod
    def post(
        self,
        data: JSON | None = None,
        params: TRequestParams | None = None,
    ) -> TModel:
        pass


class IPatchable(IMethodBase, Generic[TModel, TRequestParams]):
    def __init__(self, model: TModel) -> None:
        super().__init__(model)

    @abstractmethod
    def patch(
        self,
        data: PatchRequestData,
        params: TRequestParams | None = None,
    ) -> TModel:
        pass


class IPuttable(IMethodBase, Generic[TModel, TRequestParams]):
    def __init__(self, model: TModel) -> None:
        super().__init__(model)

    @abstractmethod
    def put(
        self,
        data: JSON | None = None,
        params: TRequestParams | None = None,
    ) -> TModel:
        pass


class IDeleteable(IMethodBase, Generic[TRequestParams]):
    def __init__(self, model: TModel) -> None:
        super().__init__(model)

    @abstractmethod
    def delete(
        self,
        data: JSON | None = None,
        params: TRequestParams | None = None,
    ) -> None:
        pass



from pysimplesat.endpoints.base.base_endpoint import SimpleSatEndpoint
from pysimplesat.interfaces import (
    IPostable,
)
from pysimplesat.models.simplesat import SurveyEmail
from pysimplesat.types import (
    JSON,
    SimpleSatRequestParams,
)


class SurveysIdEmailEndpoint(
    SimpleSatEndpoint,
    IPostable[SurveyEmail, SimpleSatRequestParams],
):
    def __init__(self, client, parent_endpoint=None) -> None:
        SimpleSatEndpoint.__init__(self, client, "email", parent_endpoint=parent_endpoint)
        IPostable.__init__(self, SurveyEmail)


    def post(self, data: JSON | None = None, params: SimpleSatRequestParams | None = None) -> SurveyEmail:
        """
        Performs a POST request against the /surveys/{id}/email endpoint.

        Parameters:
            data (dict[str, Any]): The data to send in the request body.
            params (dict[str, int | str]): The parameters to send in the request query string.
        Returns:
            SurveyEmail: The parsed response data.
        """
        return self._parse_one(SurveyEmail, super()._make_request("POST", data=data, params=params).json())



from typing import Literal, TypeAlias

from typing_extensions import NotRequired, TypedDict
from datetime import datetime

Literals: TypeAlias = str | int | float | bool
JSON: TypeAlias = dict[str, "JSON"] | list["JSON"] | Literals | None


class Patch(TypedDict):
    op: Literal["add"] | Literal["replace"] | Literal["remove"]
    path: str
    value: JSON


class SimpleSatRequestParams(TypedDict):
    created_at_min: NotRequired[datetime]
    created_at_max: NotRequired[datetime]
    updated_at_min: NotRequired[datetime]
    updated_at_min: NotRequired[datetime]
    customFieldConditions: NotRequired[str]
    page_token: NotRequired[str]
    page: NotRequired[int]
    limit: NotRequired[int]
    organization_id: NotRequired[int]
    platform: NotRequired[str]
    status: NotRequired[str]
    indicator_type: NotRequired[str]
    severity: NotRequired[str]
    platform: NotRequired[str]
    agent_id: NotRequired[str]
    type: NotRequired[str]
    entity_id: NotRequired[int]
    types: NotRequired[str]
    statuses: NotRequired[str]


GenericRequestParams: TypeAlias = dict[str, Literals]
RequestParams: TypeAlias = SimpleSatRequestParams | GenericRequestParams
PatchRequestData: TypeAlias = list[Patch]
RequestData: TypeAlias = JSON | PatchRequestData
RequestMethod: TypeAlias = Literal["GET", "POST", "PUT", "PATCH", "DELETE"]



from pysimplesat.endpoints.base.base_endpoint import SimpleSatEndpoint
from pysimplesat.endpoints.simplesat.SurveysIdEndpoint import SurveysIdEndpoint
from pysimplesat.interfaces import (
    IGettable,
)
from pysimplesat.models.simplesat import Survey
from pysimplesat.types import (
    JSON,
    SimpleSatRequestParams,
)


class SurveysEndpoint(
    SimpleSatEndpoint,
    IGettable[Survey, SimpleSatRequestParams],
):
    def __init__(self, client, parent_endpoint=None) -> None:
        SimpleSatEndpoint.__init__(self, client, "surveys", parent_endpoint=parent_endpoint)
        IGettable.__init__(self, Survey)

    def id(self, id: int) -> SurveysIdEndpoint:
        """
        Sets the ID for this endpoint and returns an initialized SurveysIdEndpoint object to move down the chain.

        Parameters:
            id (int): The ID to set.
        Returns:
            SurveysIdEndpoint: The initialized SurveysIdEndpoint object.
        """
        child = SurveysIdEndpoint(self.client, parent_endpoint=self)
        child._id = id
        return child

    def get(
        self,
        data: JSON | None = None,
        params: SimpleSatRequestParams | None = None,
    ) -> Survey:
        """
        Performs a GET request against the /surveys endpoint.

        Parameters:
            data (dict[str, Any]): The data to send in the request body.
            params (dict[str, int | str]): The parameters to send in the request query string.
        Returns:
            Survey: The parsed response data.
        """
        print("get")
        return self._parse_many(
            Survey,
            super()._make_request("GET", data=data, params=params).json().get('surveys', {}),
        )



import json
from enum import Enum
from typing import Any


class Patch:
    class PatchOp(Enum):
        """
        PatchOperation is an enumeration of the different patch operations supported
        by the SimpleSat API. These operations are ADD, REPLACE, and REMOVE.
        """

        ADD = 1
        REPLACE = 2
        REMOVE = 3

    def __init__(self, op: PatchOp, path: str, value: Any) -> None:  # noqa: ANN401
        self.op = op.name.lower()
        self.path = path
        self.value = value

    def __repr__(self) -> str:
        """
        Return a string representation of the model as a formatted JSON string.

        Returns:
            str: A formatted JSON string representation of the model.
        """
        return json.dumps(self.__dict__, default=str, indent=2)


class PatchGroup:
    def __init__(self, *patches: Patch) -> None:
        self.patches = list(patches)

    def __repr__(self) -> str:
        return str(self.patches)



from __future__ import annotations

from datetime import datetime
from typing import Any

from pydantic import Field

from pysimplesat.models.base.base_model import SimpleSatModel

class Pagination(SimpleSatModel):
    current_page: int | None = Field(default=None, alias="CurrentPage")
    current_page_count: int | None = Field(default=None, alias="CurrentPageCount")
    limit: int | None = Field(default=None, alias="Limit")
    total_count: int | None = Field(default=None, alias="TotalCount")
    next_page: int | None = Field(default=None, alias="NextPage")
    next_page_url: str | None = Field(default=None, alias="NextPageURL")
    next_page_token: str | None = Field(default=None, alias="NextPageToken")
    
class Answer(SimpleSatModel):
    id: int | None = Field(default=None, alias="Id")
    created: datetime | None = Field(default=None, alias="Created")
    modified: datetime | None = Field(default=None, alias="Modified")
    question: dict[str, Any] | None = Field(default=None, alias="Question")
    choice: str | None = Field(default=None, alias="Choice")
    choice_label: str | None = Field(default=None, alias="ChoiceLabel")
    choices: list | None = Field(default=None, alias="Choices")
    sentiment: str | None = Field(default=None, alias="Sentiment")
    comment: str | None = Field(default=None, alias="Comment")
    follow_up_answer: str | None = Field(default=None, alias="FollowUpAnswer")
    follow_up_answer_choice: str | None = Field(default=None, alias="FollowUpAnswerChoice")
    follow_up_answer_choices: list | None = Field(default=None, alias="FollowUpAnswerChoices")
    survey: dict[str, str | int] | None = Field(default=None, alias="Survey")
    published_as_testimonial: bool | None = Field(default=None, alias="PublishedAsTestimonial")
    response_id: int | None = Field(default=None, alias="ResponseId")

class Customer(SimpleSatModel):
    id: int | None = Field(default=None, alias="Id")
    external_id: str | None = Field(default=None, alias="ExternalId")
    created: datetime | None = Field(default=None, alias="Created")
    modified: datetime | None = Field(default=None, alias="Modified")
    name: str | None = Field(default=None, alias="Name")
    email: str | None = Field(default=None, alias="Email")
    company: str | None = Field(default=None, alias="Company")
    custom_attributes: dict[str, str | int] | None = Field(default=None, alias="CustomAttributes")

class TeamMember(SimpleSatModel):
    id: int | None = Field(default=None, alias="Id")
    external_id: str | None = Field(default=None, alias="ExternalId")
    created: datetime | None = Field(default=None, alias="Created")
    modified: datetime | None = Field(default=None, alias="Modified")
    name: str | None = Field(default=None, alias="Name")
    email: str | None = Field(default=None, alias="Email")
    custom_attributes: dict[str, str | int] | None = Field(default=None, alias="CustomAttributes")

class Response(SimpleSatModel):
    survey_id: int | None = Field(default=None, alias="SurveyId")
    tags: list | None = Field(default=None, alias="Tags")
    answers: list[dict[str, Any]] | None = Field(default=None, alias="Answers")
    team_members: list[dict[str, Any]] | None = Field(default=None, alias="TeamMembers")
    ticket: dict[str, Any] | None = Field(default=None, alias="Ticket")
    customer: dict[str, Any] | None = Field(default=None, alias="Customer")

class Survey(SimpleSatModel):
    id: int | None = Field(default=None, alias="Id")
    name: str | None = Field(default=None, alias="Name")
    metric: str | None = Field(default=None, alias="Metric")
    survey_token: str | None = Field(default=None, alias="SurveyToken")
    survey_type: str | None = Field(default=None, alias="SurveyType")
    brand_name: str | None = Field(default=None, alias="BrandName")

class Question(SimpleSatModel):
    id: int | None = Field(default=None, alias="Id")
    survey: dict[str, int | str] | None = Field(default=None, alias="Survey")
    order: int | None = Field(default=None, alias="Order")
    metric: str | None = Field(default=None, alias="Metric")
    text: str | None = Field(default=None, alias="Text")
    rating_scale: bool | None = Field(default=None, alias="RatingScale")
    required: bool | None = Field(default=None, alias="Required")
    choices: list[str] | None = Field(default=None, alias="Choices")
    rules: dict[str, Any] | None = Field(default=None, alias="Rules")

class CustomerBulk(SimpleSatModel):
    request_id: str | None = Field(default=None, alias="RequestId")
    detail: str | None = Field(default=None, alias="Detail")
    
class SurveyEmail(SimpleSatModel):
    detail: str | None = Field(default=None, alias="Detail")



from __future__ import annotations

import inspect
import re
from datetime import datetime
from enum import Enum
from typing import TYPE_CHECKING, Any, Generic, TypeVar

from pysimplesat.utils.naming import to_camel_case

if TYPE_CHECKING:
    from collections.abc import Callable

T = TypeVar("T")


class ValueType(Enum):
    STR = 1
    INT = 2
    DATETIME = 3


class Condition(Generic[T]):
    def __init__(self: Condition[T]) -> None:
        self._condition_string: str = ""
        self._field = ""

    def field(self: Condition[T], selector: Callable[[type[T]], Any]) -> Condition[T]:
        field = ""

        frame = inspect.currentframe()
        try:
            context = inspect.getframeinfo(frame.f_back).code_context
            caller_lines = "".join([line.strip() for line in context])
            m = re.search(r"field\s*\(([^)]+)\)", caller_lines)
            if m:
                caller_lines = m.group(1)

            field = to_camel_case("/".join(caller_lines.replace("(", "").replace(")", "").split(".")[1:]))

        finally:
            del frame

        self._condition_string += field
        return self

    def equals(self: Condition[T], value: Any) -> Condition[T]:  # noqa: ANN401
        self._condition_string += " = "
        self.__add_typed_value_to_string(value, type(value))
        return self

    def not_equals(self: Condition[T], value: Any) -> Condition[T]:  # noqa: ANN401
        self._condition_string += " = "
        self.__add_typed_value_to_string(value, type(value))
        return self

    def less_than(self: Condition[T], value: Any) -> Condition[T]:  # noqa: ANN401
        self._condition_string += " < "
        self.__add_typed_value_to_string(value, type(value))
        return self

    def less_than_or_equals(
        self: Condition[T],
        value: Any,  # noqa: ANN401
    ) -> Condition[T]:
        self._condition_string += " <= "
        self.__add_typed_value_to_string(value, type(value))
        return self

    def greater_than(self: Condition[T], value: Any) -> Condition[T]:  # noqa: ANN401
        self._condition_string += " > "
        self.__add_typed_value_to_string(value, type(value))
        return self

    def greater_than_or_equals(
        self: Condition[T],
        value: Any,  # noqa: ANN401
    ) -> Condition[T]:
        self._condition_string += " >= "
        self.__add_typed_value_to_string(value, type(value))
        return self

    def contains(self: Condition[T], value: Any) -> Condition[T]:  # noqa: ANN401
        self._condition_string += " CONTAINS "
        self.__add_typed_value_to_string(value, type(value))
        return self

    def like(self: Condition[T], value: Any) -> Condition[T]:  # noqa: ANN401
        self._condition_string += " LIKE "
        self.__add_typed_value_to_string(value, type(value))
        return self

    def in_(self: Condition[T], value: Any) -> Condition[T]:  # noqa: ANN401
        self._condition_string += " IN "
        self.__add_typed_value_to_string(value, type(value))
        return self

    def not_(self: Condition[T], value: Any) -> Condition[T]:  # noqa: ANN401
        self._condition_string += " NOT "
        self.__add_typed_value_to_string(value, type(value))
        return self

    def __add_typed_value_to_string(  # noqa: ANN202
        self: Condition[T],
        value: Any,  # noqa: ANN401
        type: type,  # noqa: A002
    ):
        if type is str:
            self._condition_string += f'"{value}"'
        elif type is int:  # noqa: SIM114
            self._condition_string += str(value)
        elif type is bool:
            self._condition_string += str(value)
        elif type is datetime:
            self._condition_string += f"[{value}]"
        else:
            self._condition_string += f'"{value}"'

    def and_(self: Condition[T], selector: Callable[[type[T]], Any] | None = None) -> Condition[T]:
        self._condition_string += " AND "

        if selector is not None:
            field = ""
            frame = inspect.currentframe()
            try:
                context = inspect.getframeinfo(frame.f_back).code_context
                caller_lines = "".join([line.strip() for line in context])
                m = re.search(r"and_\s*\(([^)]+)\)", caller_lines)
                if m:
                    caller_lines = m.group(1)

                field = "/".join(caller_lines.replace("(", "").replace(")", "").split(".")[1:])

            finally:
                del frame

            self._condition_string += field
        return self

    def or_(self: Condition[T], selector: Callable[[type[T]], Any] | None = None) -> Condition[T]:
        self._condition_string += " OR "

        if selector is not None:
            field = ""
            frame = inspect.currentframe()
            try:
                context = inspect.getframeinfo(frame.f_back).code_context
                caller_lines = "".join([line.strip() for line in context])
                m = re.search(r"or_\s*\(([^)]+)\)", caller_lines)
                if m:
                    caller_lines = m.group(1)

                field = "/".join(caller_lines.replace("(", "").replace(")", "").split(".")[1:])

            finally:
                del frame

            self._condition_string += field
        return self

    def wrap(self: Condition[T], condition: Callable[[Condition[T]], Condition[T]]) -> Condition[T]:
        self._condition_string += f"({condition(Condition[T]())})"
        return self

    def __str__(self: Condition[T]) -> str:
        return self._condition_string.strip()



from pysimplesat.endpoints.base.base_endpoint import SimpleSatEndpoint
from pysimplesat.endpoints.simplesat.CustomersIdEndpoint import CustomersIdEndpoint
from pysimplesat.endpoints.simplesat.CustomersBulkEndpoint import CustomersBulkEndpoint
from pysimplesat.interfaces import (
    IPostable,
)
from pysimplesat.models.simplesat import Customer
from pysimplesat.types import (
    JSON,
    SimpleSatRequestParams,
)


class CustomersEndpoint(
    SimpleSatEndpoint,
    IPostable[Customer, SimpleSatRequestParams],
):
    def __init__(self, client, parent_endpoint=None) -> None:
        SimpleSatEndpoint.__init__(self, client, "customers", parent_endpoint=parent_endpoint)
        IPostable.__init__(self, Customer)
        self.bulk = self._register_child_endpoint(CustomersBulkEndpoint(client, parent_endpoint=self))

    def id(self, id: int) -> CustomersIdEndpoint:
        """
        Sets the ID for this endpoint and returns an initialized CustomersIdEndpoint object to move down the chain.

        Parameters:
            id (int): The ID to set.
        Returns:
            CustomersIdEndpoint: The initialized CustomersIdEndpoint object.
        """
        child = CustomersIdEndpoint(self.client, parent_endpoint=self)
        child._id = id
        return child

    def post(self, data: JSON | None = None, params: SimpleSatRequestParams | None = None) -> Customer:
        """
        Performs a POST request against the /customers endpoint.

        Parameters:
            data (dict[str, Any]): The data to send in the request body.
            params (dict[str, int | str]): The parameters to send in the request query string.
        Returns:
            Customer: The parsed response data.
        """
        return self._parse_one(Customer, super()._make_request("POST", data=data, params=params).json())



from pysimplesat.endpoints.base.base_endpoint import SimpleSatEndpoint
from pysimplesat.endpoints.simplesat.AnswersIdEndpoint import AnswersIdEndpoint
from pysimplesat.endpoints.simplesat.AnswersSearchEndpoint import AnswersSearchEndpoint


class AnswersEndpoint(
    SimpleSatEndpoint,
):
    def __init__(self, client, parent_endpoint=None) -> None:
        SimpleSatEndpoint.__init__(self, client, "answers", parent_endpoint=parent_endpoint)
        self.search = self._register_child_endpoint(AnswersSearchEndpoint(client, parent_endpoint=self))

    def id(self, id: int) -> AnswersIdEndpoint:
        """
        Sets the ID for this endpoint and returns an initialized AnswersIdEndpoint object to move down the chain.

        Parameters:
            id (int): The ID to set.
        Returns:
            AnswersIdEndpoint: The initialized AnswersIdEndpoint object.
        """
        child = AnswersIdEndpoint(self.client, parent_endpoint=self)
        child._id = id
        return child



from pysimplesat.endpoints.base.base_endpoint import SimpleSatEndpoint
from pysimplesat.endpoints.simplesat.SurveysIdEmailEndpoint import SurveysIdEmailEndpoint


class SurveysIdEndpoint(
    SimpleSatEndpoint,
):
    def __init__(self, client, parent_endpoint=None) -> None:
        SimpleSatEndpoint.__init__(self, client, "{id}", parent_endpoint=parent_endpoint)
        self.email = self._register_child_endpoint(SurveysIdEmailEndpoint(client, parent_endpoint=self))



import typing
from datetime import datetime, timezone

from pysimplesat.clients.base_client import SimpleSatClient
from pysimplesat.config import Config

if typing.TYPE_CHECKING:
    from pysimplesat.endpoints.simplesat.SurveysEndpoint import SurveysEndpoint
    from pysimplesat.endpoints.simplesat.AnswersEndpoint import AnswersEndpoint
    from pysimplesat.endpoints.simplesat.CustomersEndpoint import CustomersEndpoint
    from pysimplesat.endpoints.simplesat.QuestionsEndpoint import QuestionsEndpoint
    from pysimplesat.endpoints.simplesat.TeamMembersEndpoint import TeamMembersEndpoint
    from pysimplesat.endpoints.simplesat.ResponsesEndpoint import ResponsesEndpoint


class SimpleSatAPIClient(SimpleSatClient):
    """
    SimpleSat API client. Handles the connection to the SimpleSat API
    and the configuration of all the available endpoints.
    """

    def __init__(
        self,
        privatekey: str,
    ) -> None:
        """
        Initializes the client with the given credentials.

        Parameters:
            privatekey (str): Your SimpleSat API private key.
        """
        self.privatekey: str = privatekey
        self.token_expiry_time: datetime = datetime.now(tz=timezone.utc)

    # Initializing endpoints
    @property
    def surveys(self) -> "SurveysEndpoint":
        from pysimplesat.endpoints.simplesat.SurveysEndpoint import SurveysEndpoint

        return SurveysEndpoint(self)

    @property
    def answers(self) -> "AnswersEndpoint":
        from pysimplesat.endpoints.simplesat.AnswersEndpoint import AnswersEndpoint

        return AnswersEndpoint(self)

    @property
    def customers(self) -> "CustomersEndpoint":
        from pysimplesat.endpoints.simplesat.CustomersEndpoint import CustomersEndpoint

        return CustomersEndpoint(self)

    @property
    def questions(self) -> "QuestionsEndpoint":
        from pysimplesat.endpoints.simplesat.QuestionsEndpoint import QuestionsEndpoint

        return QuestionsEndpoint(self)

    @property
    def team_members(self) -> "TeamMembersEndpoint":
        from pysimplesat.endpoints.simplesat.TeamMembersEndpoint import TeamMembersEndpoint

        return TeamMembersEndpoint(self)

    @property
    def responses(self) -> "ResponsesEndpoint":
        from pysimplesat.endpoints.simplesat.ResponsesEndpoint import ResponsesEndpoint

        return ResponsesEndpoint(self)


    def _get_url(self) -> str:
        """
        Generates and returns the URL for the SimpleSat API endpoints based on the company url and codebase.
        Logs in an obtains an access token.
        Returns:
            str: API URL.
        """
        return f"https://api.simplesat.io/api/v1"

    def _get_headers(self) -> dict[str, str]:
        """
        Generates and returns the headers required for making API requests. The access token is refreshed if necessary before returning.

        Returns:
            dict[str, str]: Dictionary of headers including Content-Type, Client ID, and Authorization.
        """
        return {
            "Content-Type": "application/json",
            "X-Simplesat-Token": f"{self.privatekey}",
        }



from pysimplesat.endpoints.base.base_endpoint import SimpleSatEndpoint
from pysimplesat.interfaces import (
    IPostable,
    IPaginateable,
)
from pysimplesat.models.simplesat import Response
from pysimplesat.responses.paginated_response import PaginatedResponse
from pysimplesat.types import (
    JSON,
    SimpleSatRequestParams,
)


class ResponsesSearchEndpoint(
    SimpleSatEndpoint,
    IPostable[Response, SimpleSatRequestParams],
    IPaginateable[Response, SimpleSatRequestParams],

):
    def __init__(self, client, parent_endpoint=None) -> None:
        SimpleSatEndpoint.__init__(self, client, "search", parent_endpoint=parent_endpoint)
        IPostable.__init__(self, Response)
        IPaginateable.__init__(self, Response)

    def paginated(
        self,
        page: int,
        limit: int,
        params: SimpleSatRequestParams | None = None,
    ) -> PaginatedResponse[Response]:
        """
        Performs a POST request against the /responses/search endpoint and returns an initialized PaginatedResponse object.

        Parameters:
            page (int): The page number to request.
            limit (int): The number of results to return per page.
            params (dict[str, int | str]): The parameters to send in the request query string.
        Returns:
            PaginatedResponse[Response]: The initialized PaginatedResponse object.
        """
        if params:
            params["page[number]"] = page
            params["page[size]"] = limit
        else:
            params = {"page[number]": page, "page[size]": limit}
        return PaginatedResponse(
            super()._make_request("POST", params=params),
            Response,
            self,
            "responses",
            page,
            limit,
            params,
        )


    #TODO: How do I paginate a post?
    def post(self, data: JSON | None = None, params: SimpleSatRequestParams | None = None) -> Response:
        """
        Performs a POST request against the /responses/search endpoint.

        Parameters:
            data (dict[str, Any]): The data to send in the request body.
            params (dict[str, int | str]): The parameters to send in the request query string.
        Returns:
            Survey: The parsed response data.
        """
        return self._parse_many(Response, super()._make_request("POST", data=data, params=params).json().get('responses', {}))
